class CorpusAnalysisApp {
  constructor(corpusId) {
    this.corpusId = corpusId;

    this.data = {};

    // HTML elements
    this.elements = {
      container: document.querySelector('#corpus-analysis-app-container'),
      extensionCards: document.querySelector('#corpus-analysis-app-extension-cards'),
      extensionTabs: document.querySelector('#corpus-analysis-app-extension-tabs'),
      initModal: document.querySelector('#corpus-analysis-app-init-modal')
    };
    // Materialize elements
    this.elements.m = {
      extensionTabs: M.Tabs.init(this.elements.extensionTabs),
      initModal: M.Modal.init(this.elements.initModal, {dismissible: false})
    };

    this.extensions = {};

    this.settings = {};
  }

  async init() {
    this.disableActionElements();
    this.elements.m.initModal.open();

    // Setup CQi over SocketIO connection and gather data from the CQPServer
    try {
      const cqiClient = new cqi.CQiClient('/cqi_over_sio');
      const response = await cqiClient.api.socket.emitWithAck('init', this.corpusId);
      if (response.code !== 200) {throw new Error();}
      await cqiClient.connect('anonymous', '');
      const cqiCorpus = await cqiClient.corpora.get(`NOPAQUE-${this.corpusId.toUpperCase()}`);
      // TODO: Don't do this hgere
      await cqiCorpus.updateDb();
      this.data.cqiClient = cqiClient;
      this.data.cqiCorpus = cqiCorpus;
      this.data.corpus = {o: cqiCorpus};  // legacy
    } catch (error) {
      // TODO: Currently we can only handle CQiErrors here,
      //       but we should also handle other errors.
      const errorString = `${error.code}: ${error.constructor.name}`;
      const errorsElement = this.elements.initModal.querySelector('.errors');
      const progressElement = this.elements.initModal.querySelector('.progress');
      errorsElement.innerText = errorString;
      errorsElement.classList.remove('hide');
      progressElement.classList.add('hide');
      return;
    }

    // Initialize extensions
    for (const extension of Object.values(this.extensions)) {extension.init();}
    for (const extensionSelectorElement of this.elements.extensionCards.querySelectorAll('.extension-selector')) {
      extensionSelectorElement.addEventListener('click', () => {
        this.elements.m.extensionTabs.select(extensionSelectorElement.dataset.target);
      });
    }

    this.enableActionElements();
    this.elements.m.initModal.close();
  }

  registerExtension(extension) {
    if (extension.name in this.extensions) {return;}
    this.extensions[extension.name] = extension;
  }

  disableActionElements() {
    const actionElements = this.elements.container.querySelectorAll('.corpus-analysis-action');
    for (const actionElement of actionElements) {
      switch(actionElement.nodeName) {
        case 'INPUT':
          actionElement.disabled = true;
          break;
        case 'SELECT':
          actionElement.parentNode.querySelector('input.select-dropdown').disabled = true;
          break;
        default:
          actionElement.classList.add('disabled');
      }
    }
  }

  enableActionElements() {
    const actionElements = this.elements.container.querySelectorAll('.corpus-analysis-action');
    for (const actionElement of actionElements) {
      switch(actionElement.nodeName) {
        case 'INPUT':
          actionElement.disabled = false;
          break;
        case 'SELECT':
          actionElement.parentNode.querySelector('input.select-dropdown').disabled = false;
          break;
        default:
          actionElement.classList.remove('disabled');
      }
    }
  }
}
